package net.pricefx.common.apibuilder;

import java.util.Arrays;
import java.util.Collections;
import java.util.List;

public final class MethodInvocation {

	private MethodInvocation(String methodNameToken, String methodNameId,  Object ... arguments) {
		_methodNameToken = methodNameToken;
		_methodNameId = methodNameId;
		if (arguments == null || arguments.length == 0) 
			_arguments = Collections.emptyList();
		else
			_arguments = Collections.unmodifiableList(Arrays.asList(arguments));
	}
	
	private final String _methodNameToken;
	private String _methodNameId;
	private final List<Object> _arguments;
	
	@Override public String toString() { return _methodNameToken; }

	public String methodNameToken() { return _methodNameToken; }
	public String methodNameId() { return _methodNameId; }
	public List<Object> arguments() {return _arguments; }
	
	public static MethodInvocation of(String methodNameToken, String methodNameId,  Object ... arguments) {
		return new MethodInvocation(methodNameToken, methodNameId, arguments);
	}

	@Override
	public int hashCode() {
		final int prime = 31;
		int result = 1;
		result = prime * result + ((_methodNameId == null) ? 0 : _methodNameId.hashCode());
		return result;
	}

	@Override
	public boolean equals(Object obj) {
		if (this == obj) return true;
		if (obj == null) return false;
		if (getClass() != obj.getClass()) return false;
		MethodInvocation other = (MethodInvocation) obj;
		if (_methodNameId == null) {
			if (other._methodNameId != null) return false;
		} else if (!_methodNameId.equals(other._methodNameId)) return false;
		return true;
	}
	
}