package net.pricefx.common.apibuilder;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.Date;
import java.util.Iterator;
import java.util.List;
import java.util.Map;

public class Methods {

	public String createMethodNameToken(ObjectIdentity objectIdentity, String methodName) {
		ObjectIdentity objIdentity = objectIdentity;
		String methodNameToken = methodName; 
		while ( objIdentity != ObjectIdentity.NONE) {
			methodNameToken = objIdentity.classId + "." + methodNameToken;
			objIdentity = objIdentity.parentObjectIdentity;
		}
		return methodNameToken;
	}

	public String createMethodId(ObjectIdentity objectIdentity, String methodName) {
		ObjectIdentity objIdentity = objectIdentity;
		String methodId = methodName; 
		while ( objIdentity != ObjectIdentity.NONE) {
			methodId = objIdentity.instanceId + "." + methodId;
			objIdentity = objIdentity.parentObjectIdentity;
		}
		return methodId;
	}
	
	public String createMethodName(String methodName, List<Object> arguments) {
		if (arguments == null || arguments.isEmpty()) return methodName + "()";
		
		String methodString = methodName + "(";
		for (int i = 0; i < arguments.size(); i++) {
			methodString += createMethodArgument(arguments.get(i));
			if (i < arguments.size() - 1) methodString += ", ";
		}
		methodString += ")";
		
		return methodString;
	}
	
	private String createMethodArgument(Object arg) {
		if(arg == null) {
			return "null"; // displaying also nulls
		} else if (arg instanceof String) {
			arg = ((String) arg).replace("'", "\\'");
			// we might consider escaping slashes as well
			return "'" + arg + "'";
		} else if (arg instanceof Date) {
			return "new Date(" + ((Date) arg).getTime() + ")";
		} else if (BuilderExtensions.isDefaultImport(arg)) {
			String fullName = arg.getClass().getName();
			return fullName.substring(fullName.lastIndexOf(".") +1) + "." + arg.toString();
		} else if (arg.getClass().isArray()) {
			String argString = "[";
			List<Object> arrAsList = Arrays.asList((Object[]) arg);
			int length = /*Array.getLength(arg) - doesn't work in gwt*/arrAsList.size();
			for (int i = 0; i < length; i++) {
                argString += createMethodArgument(/*Array.get(arg, i) - doesn't work in gwt*/arrAsList.get(i));
                if (i < length - 1) argString += ", ";
            }
			argString += "]";
			return argString + " as " + calcName(arg.getClass());
		} else if (arg instanceof Iterable) {
			String argString = "[";
			for (Iterator<?> it = ((Iterable<?>) arg).iterator(); it.hasNext();) {
				argString += createMethodArgument(it.next());
				if (it.hasNext()) argString += ", ";
			}
			argString += "]";
			return argString + ((arg instanceof ArrayList) ? "" : " as " + calcName(arg.getClass()));
		} else if (arg instanceof Map) {
			String argString = "[";
			for (Iterator<?> it = ((Map<?, ?>) arg).entrySet().iterator(); it.hasNext();) {
				Map.Entry<?, ?> argItem = (Map.Entry<?, ?>) it.next();
				argString += createMethodArgument(argItem.getKey()) + ": " + createMethodArgument(argItem.getValue());
				if (it.hasNext()) argString += ", ";
			}
			argString += "]";
			// the resulting map will always be a LinkedHashMap - the caller must be aware of that; i.e. we're not throwing/logging anything, here
			return argString;
		} else {
			return arg.toString();
		}
	}
	
	private String calcName(Class<?> cls) {
        if (cls.isArray()) {
            return calcName(cls.getComponentType())+"[]";
        }
        return cls.getName();
    }
	
}