package net.pricefx.common.apibuilder.calculationflowaction;

import net.pricefx.common.apibuilder.Invocations;

/**
 * This action starts calculation of selected data load. It's a programmatic version
 * of the same action available in UI. As such it allows user to schedule the
 * action in calculation flow formula.
 */
public class DataLoadAction<ATTACHEDTO extends Help> extends Action {

	private String dataLoadLabel;
	private String dataLoadType;
	private String target;
	private Boolean calculate;
	private ATTACHEDTO backLink;
	
	public DataLoadAction(ATTACHEDTO backLink, Invocations<MethodInvocationRecord> invocations, String dataLoadLabel, String dataLoadType, String target) {
		super(invocations);
		this.backLink = backLink;
		this.dataLoadLabel = dataLoadLabel;
		this.dataLoadType = dataLoadType;
		this.target = target;
		this.calculate = false;
	}

	/**
	 * Whether the action is valid and should be triggered or not.
	 * 
	 * @param calculate
	 *            <ul>
	 *            <li>true - calculation of the data load is started as background
	 *            process</li>
	 *            <li>false - calculation of the data load is NOT started at all</li>
	 *            </ul>
	 */
	public DataLoadAction<ATTACHEDTO> setCalculate(Boolean calculate) {
		invocations.add(MethodInvocationRecord.of(getObjectName() + ".setCalculate", calculate));
		this.calculate = Boolean.TRUE.equals(calculate) ? true : false;
		return this;
	}

	public ATTACHEDTO back() {
		invocations.add(MethodInvocationRecord.of(getObjectName() + ".back"));
		return backLink;
	}

	@Override
	public String getObjectName() {
		return backLink.getObjectName() + ".DataLoadAction";
	}

	public String getDataLoadLabel() { return dataLoadLabel; }
	public String getDataLoadType() { return dataLoadType;	}
	public String getTarget() { return target; }
	public Boolean getCalculate() {	return calculate; }

	@Override
	public String toString() {
		return "DataLoadAction [dataLoadLabel=" + dataLoadLabel	+ ", dataLoadType=" + dataLoadType + ", target=" + target
				+ ", calculate=" + calculate + ", backLink=" + backLink	+ ", hashCode()=" + hashCode() + "]";
	}

}