package net.pricefx.common.apibuilder.calculationflowaction;

import net.pricefx.common.apibuilder.Invocations;

import java.util.Date;

/**
 * This action starts calculation of selected manual pricelist. It's a programmatic version
 * of the same action available in UI. As such it allows user to schedule the
 * action in calculation flow formula.
 */
public class ManualPricelistAction<ATTACHEDTO extends Help> extends Action {

	private String manualPricelistName;
	private Boolean calculate;
	private Date validAfter;
	private int priority;
	private ATTACHEDTO backLink;
	private String simulationSet;
	
	public ManualPricelistAction(ATTACHEDTO backLink, Invocations<MethodInvocationRecord> invocations, String manualPricelistName) {
		super(invocations);
		this.backLink = backLink;
		this.manualPricelistName = manualPricelistName;
		this.calculate = false;
		this.validAfter = null;
	}

	/**
	 * Whether the action is valid and should be triggered or not.
	 * 
	 * @param calculate
	 *            <ul>
	 *            <li>true - calculation of the manual pricelist is started as background
	 *            process</li>
	 *            <li>false - calculation of the manual pricelist is NOT started at all</li>
	 *            </ul>
	 */
	public ManualPricelistAction<ATTACHEDTO> setCalculate(Boolean calculate) {
		invocations.add(MethodInvocationRecord.of(getObjectName() + ".setCalculate", calculate));
		this.calculate = Boolean.TRUE.equals(calculate);
		return this;
	}	
	
	/**
	 * Informs the runtime to work with MPL that is valid after the specified date.
	 * 
	 * @param validAfter <code<null</code> value can be used to work with current date
	 * @return this
	 */
	public ManualPricelistAction<ATTACHEDTO> setValidAfter(Date validAfter) {
		invocations.add(MethodInvocationRecord.of(getObjectName() + ".setValidAfter", validAfter));
		this.validAfter = validAfter;
		return this;
	}	
	
	/**
	 * Sets the prority of this job. 
	 * 
	 * @param priority Default is 0 (lowest user-definable priority). Max value is 9.
	 * @return this
	 */
	public ManualPricelistAction<ATTACHEDTO> setPriority(int priority) {
		invocations.add(MethodInvocationRecord.of(getObjectName() + ".setPriority", priority));
		this.priority = priority;
		return this;		
	}	
	
	/**
	 * Sets simulation set. Simulation set identifies together with manual
	 * pricelist name the pricelist we are looking for.
	 * 
	 * @param simulationSet
	 *            simulation set or {@code null} if there is no such simulation
	 *            set attached
	 */
	public ManualPricelistAction<ATTACHEDTO> setSimulationSet(String simulationSet) {
		invocations.add(MethodInvocationRecord.of(getObjectName() + ".setSimulationSet", simulationSet));
		this.simulationSet = simulationSet;
		return this;
	}
	
	public ATTACHEDTO back() {
		invocations.add(MethodInvocationRecord.of(getObjectName() + ".back"));
		return backLink;
	}

	@Override
	public String getObjectName() {
		return backLink.getObjectName() + ".ManualPricelistAction";
	}

	public String getManualPricelistName() { return manualPricelistName; }
	public String getSimulationSet() { return simulationSet; }
	public Boolean getCalculate() {	return calculate; }
	public Date getValidAfter() {	return validAfter; }
	public int getPriority() { return priority; }
	
	@Override public String toString() { return "ManualPricelistAction [manualPricelistName=" + manualPricelistName + ", calculate=" + calculate
				+ ", backLink=" + backLink + ", simulationSet=" + simulationSet	+ ", hashCode()=" + hashCode() + "]";
	}
	
	
}