package net.pricefx.common.apibuilder.chart;

import net.pricefx.common.api.chart.FilterOperator;
import net.pricefx.common.apibuilder.BacklinkBuilder;
import net.pricefx.common.apibuilder.ObjectIdentity;

public class Filters<O extends ObjectIdentity> extends BacklinkBuilder<O> {

	public Filters(final O backLink, final String instanceId, final String classId) {
		super(instanceId, classId, backLink);
	}

	public Filters<O> addFilter(final String fieldName, final FilterOperator filterOperator) {
		addInvocation("addFilter",getRootFiltersInstance(),fieldName, filterOperator);
		return this;
	}
	
	public Filters<O> addFilter(final String fieldName, final FilterOperator filterOperator, final Object value) {
		addInvocation("addFilter",getRootFiltersInstance(),fieldName, filterOperator, value);
		return this;
	}

	/**
	 * Adds a (possibly complex) filter.
	 * <p>
	 * At the moment, the passed object can only be a map conforming to the
	 * representation defined by SmartGwt's <code>Criteria</code> classes (i.e.,
	 * will contain keys like <code>_constructor</code>, <code>operator</code>,
	 * <code>criteria</code>, or <code>fieldName</code>, <code>value</code> and
	 * so forth.
	 * 
	 * @see com.smartgwt.client.data.Criteria
	 * @see com.smartgwt.client.data.AdvancedCriteria
	 */
	public Filters<O> addFilter(final Object advancedFilter) {
		addInvocation("addFilter",getRootFiltersInstance(),advancedFilter);
		return this;
	}
	
	public Filters<Filters<O>> addFilters(final FilterOperator operator) {
		final String id = addInvocation("addFilters",getRootFiltersInstance(),operator);
		return new Filters<>(this, id, "Filters");
	}
	
	private Filters<?> getRootFiltersInstance() {
		Filters<?> result = this;
		while (result.getParent() instanceof Filters) {
			result = (Filters<?>) result.getParent();
		}
		return result;
	}

}