package net.pricefx.common.apibuilder.clicoll;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.LinkedHashMap;
import java.util.LinkedList;
import java.util.List;
import java.util.Map;
import java.util.Map.Entry;


public class LineItemHelper {

	protected final Map<String, Object> calculableLineItemCollection;
	protected final Map<String, List<Map<String, Object>>> parentIdToLineItems;
	protected final Map<String, Object> idToLineItem;
	protected final LineItem root;
	
	@SuppressWarnings("unchecked")
	public LineItemHelper(Map<String, Object> calculableLineItemCollection) {
		this.calculableLineItemCollection = calculableLineItemCollection;
		
		List<Map<String, Object>> lineItems = (List<Map<String, Object>>) calculableLineItemCollection.get("lineItems");
		if (lineItems == null) lineItems = new ArrayList<>();
		
		parentIdToLineItems = new LinkedHashMap<>();
		for (Map<String, Object> lineItem : lineItems) {
			String parentId = (String)lineItem.get("parentId");
			List<Map<String, Object>> children = parentIdToLineItems.get(parentId);
			if (children == null) {
				children = new ArrayList<>();
				parentIdToLineItems.put(parentId, children);
			}
			children.add(lineItem);
		}
		
		idToLineItem = new HashMap<>();
		for (Map<String, Object> lineItem : lineItems) {
			idToLineItem.put((String)lineItem.get("lineId"), lineItem);
		}
		
		Map<String, Object> rootLineItemMap = new LinkedHashMap<>();
		root = new LineItem(this, calculableLineItemCollection, rootLineItemMap) {
			@Override
			public boolean isRoot() {
				return true;
			}
			@Override
			public boolean isFolder() {
				// mimicking UI
				return true;
			}
			@Override
			public String getLabel() {
				return (String)  calculableLineItemCollectionMap.get("label");
			}
			@Override
			public LineItem getParent() {
				return null;
			}
			@Override
			public String getLineId() {
				return null;
			}
			@Override
			public String getSKU() {
				return null;
			}
			@Override
			public List<LineItem> getSiblings() {
				return new ArrayList<>();
			}
			@Override
			public List<Map<String, Object>> getInputs() {
				return (List<Map<String, Object>>)  calculableLineItemCollectionMap.get("inputs");
			}
			@Override
			public List<Map<String, Object>> getOutputs() {
				return (List<Map<String, Object>>)  calculableLineItemCollectionMap.get("outputs");
			}
			
		};
		
	}

	public LineItem lineItemFromMap(Map<String, Object> lineItemMap) {
		LineItem lineItem = new LineItem(this, calculableLineItemCollection, lineItemMap);
		return lineItem;
	}
	
	@SuppressWarnings("unchecked")
	public LineItem findByLineId(String lineItemId) {
		if(lineItemId == null) return null;
		if("ROOT".equals(lineItemId)) return root;
		
		Map<String, Object> map = (Map<String, Object>) idToLineItem.get(lineItemId);
		if (map == null) return null;
		
		LineItem lineItem = new LineItem(this, calculableLineItemCollection, map);
		return lineItem;
	}
	
	public LineItem getRoot() {
		return root;
	}
	
	@SuppressWarnings("unchecked")
	public List<LineItem> findAllWithLabel(String label) {
		ArrayList<LineItem> is = new ArrayList<>();
		for (Entry<String, Object> e : idToLineItem.entrySet()) {
			Map<String, Object> lineItemMap = (Map<String, Object>) e.getValue();
			if (label.equals(lineItemMap.get("treeLabel"))) {
				is.add(lineItemFromMap(lineItemMap));
			}
		}
		
		if (label.equals(root.getLabel())) {
			is.add(root);
		}
		
		return is;
	}
	
	public static class LineItem {

		private LineItem(LineItem i, Map<String, Object> iMap) {
			this.helper = i.helper;
			this.calculableLineItemCollectionMap = i.calculableLineItemCollectionMap;
			this.lineItemMap = iMap;
		}
		

		private LineItem(LineItemHelper helper, Map<String, Object> calculableLineItemCollectionMap, Map<String, Object> lineItemMap) {
			this.helper = helper;
			this.calculableLineItemCollectionMap = calculableLineItemCollectionMap;
			this.lineItemMap = lineItemMap;
		}

		protected LineItemHelper helper;
		protected Map<String, Object> calculableLineItemCollectionMap;
		protected Map<String, Object> lineItemMap;

		public Map<String, Object> asMap() {
			return lineItemMap;
		}
		
		public boolean isRoot() {
			return false;
		}
		
		public boolean isFolder() {
			return Boolean.TRUE.equals(lineItemMap.get("folder"));
		}
		
		public String getLabel() {
			return (String)  lineItemMap.get("treeLabel");
		}
		
		public String getLineId() {
			return (String)  lineItemMap.get("lineId");
		}
		
		public String getSKU() {
			return (String)  lineItemMap.get("sku");
		}
		
		public String getRebateTypeUN() {
			return (String)  lineItemMap.get("rebateType");
		}
		
		public String getContractTermTypeUN() {
			return (String)  lineItemMap.get("contractTermType");
		}
		
		public List<LineItem> getChildren() {
			ArrayList<LineItem> ls = new ArrayList<>();
			List<Map<String, Object>> children = helper.parentIdToLineItems.get(getLineId());
			if (children != null) {
				for (Map<String, Object> child : children)
					ls.add(helper.lineItemFromMap(child));
			}
			return ls;
		}
		
		public List<LineItem> getSiblings() {
			ArrayList<LineItem> childrenExceptMe = new ArrayList<>();
			for (LineItem i : getParent().getChildren()) {
				if (!getLineId().equals(i.getLineId())) {
					childrenExceptMe.add(i);
				}
			}
			return childrenExceptMe;
		}
		
		@SuppressWarnings("unchecked")
		public List<Map<String, Object>> getInputs() {
			return (List<Map<String, Object>>)  lineItemMap.get("inputs");
		}
		
		public Map<String, Object> getInputByName(String inputName) {
			List<Map<String, Object>> inputs = getInputs();
			if (inputs == null) return null;
			for (Map<String, Object> i : inputs) {
				if (inputName != null && inputName.equals(i.get("name"))) {
					return i;
				}
			}
			return null;
		}
		
		public Map<String, Object> getOutputByName(String outputName) {
			List<Map<String, Object>> outputs = getOutputs();
			if (outputs == null) return null;			
			for (Map<String, Object> i : outputs) {
				if (outputName != null && outputName.equals(i.get("resultName"))) {
					return i;
				}
			}
			return null;
		}
		
		@SuppressWarnings("unchecked")
		public List<Map<String, Object>> getOutputs() {
			return (List<Map<String, Object>>)  lineItemMap.get("outputs");
		}
		
		@SuppressWarnings("unchecked")
		public LineItem getParent() {
			String parentId = (String) lineItemMap.get("parentId");
			Map<String, Object> parent = (Map<String, Object>) helper.idToLineItem.get(parentId);
			if (parent == null) return helper.root;
			
			LineItem l = new LineItem(this, parent);
			return l;
		}
		
		public List<LineItem> getSubLineItems() {
			LinkedList<LineItem> walkThrough = new LinkedList<>();
			ArrayList<LineItem> lineItems = new ArrayList<>();
			List<Map<String, Object>> children = helper.parentIdToLineItems.get(getLineId());
			if (children != null) {
				for (Map<String, Object> child : children)
					walkThrough.add(helper.lineItemFromMap(child));
			}

			LineItem item = walkThrough.poll();
			while (item != null) {
				lineItems.add(item);
				List<LineItem> chldrn = item.getChildren();
				walkThrough.addAll(chldrn);
				item = walkThrough.poll();
			}
			
			return lineItems;
		}
		
	}
	
}
