package net.pricefx.common.apibuilder.contract;

import java.util.Map;

import net.pricefx.common.api.HeaderPhase;
import net.pricefx.common.apibuilder.IdGenerator;
import net.pricefx.common.apibuilder.Invocations;
import net.pricefx.common.apibuilder.MethodInvocation;
import net.pricefx.common.apibuilder.ObjectIdentity;
import net.pricefx.common.apibuilder.clicoll.CalculableLineItemCollectionBuilder;
import net.pricefx.domain.Contract;

/**
 * 
 *  IMPORTANT: Please read super class notes on sequence of execution!
 *
 */
public class ContractBuilder extends CalculableLineItemCollectionBuilder<ContractBuilder> {

	// read only state, don't get propagated further, it is here for user convenience
	private final Object contractView;
	private final ContractHelper helper;
	
	@SuppressWarnings("unchecked")
	ContractBuilder(String instanceId, String classId, ObjectIdentity parentObjectIdentity, IdGenerator idGenerator, Invocations<MethodInvocation> invocations, Object contractView, HeaderPhase phase) {
		super(instanceId, classId, idGenerator, invocations, phase);
		this.contractView = contractView;
		this.helper = new ContractHelper((Map<String, Object>) contractView);
	}


	// This has been modified in Studio because of auto-completion
	public Contract getContractView() {
		addInvocation("getContractView");
		return (Contract) contractView;
	}
	
	public ContractHelper getHelper() {
		addInvocation("getHelper");		
		return helper;
	}	
	
	@Override
	public ContractBuilderResult build() {
		addInvocation(METHOD_BUILD);

		ContractBuilderResult r = new ContractBuilderResult();
		r.invocations = invocations.get();
		return r;
	}

}