package net.pricefx.common.dto;

import com.fasterxml.jackson.annotation.JsonIgnore;
import com.fasterxml.jackson.annotation.JsonIgnoreProperties;

import java.util.HashMap;
import java.util.List;
import java.util.Map;

@JsonIgnoreProperties(ignoreUnknown = true)
public class ChainedJobInfo {
    private boolean executeOnError;
    private List<ChainedJobInfo> nextJobs;
    private String targetObject;
    private Map<String,Object> parameters;

    public List<ChainedJobInfo> getNextJobs() {
        return nextJobs;
    }

    /**
     * Sets a list of jobs that are then executed after this one.
     * @param nextJobs
     */
    public void setNextJobs(List<ChainedJobInfo> nextJobs) {
        this.nextJobs = nextJobs;
    }

    public String getTargetObject() {
        return targetObject;
    }

    /**
     * Sets the target object of next job. For example the TypedId of a PriceGrid or a CFS
     * @param targetObject
     */
    public void setTargetObject(String targetObject) {
        this.targetObject = targetObject;
    }

    public Map<String, Object> getParameters() {
        return parameters;
    }

    /**
     * Sets the additional init parameters of this next job. OCntents vary by job and target object type
     * @param parameters
     */
    public void setParameters(Map<String, Object> parameters) {
        this.parameters = parameters;
    }

    public boolean isExecuteOnError() {
        return executeOnError;
    }

    /**
     * Should this job be executed even if the previous job failed?
     * @param executeOnError
     */
    public void setExecuteOnError(boolean executeOnError) {
        this.executeOnError = executeOnError;
    }

    @JsonIgnore
    public Map<String, Object> getCombinedParameters(String key) {
        if(nextJobs == null && parameters == null)
            return null;

        Map<String,Object> combinedP = new HashMap<>();
        if(parameters != null)
            combinedP.putAll(parameters);

        if(nextJobs != null)
            combinedP.put(key,nextJobs);

        return combinedP;
    }
}
