package net.pricefx.formulaengine;

import java.io.Serializable;
import java.util.Collection;
import java.util.Map;

import net.pricefx.formulaengine.scripting.Matrix2D;

/**
 * Provides access to a QueryResult in a formula context.
 * A query result may be obtained by 
 * 		- executing a {@link net.pricefx.server.util.metamodel.dm.Query Query} in the {@link net.pricefx.formulaengine.DatamartContext DatamartContext}
 * 		- calling the DatamartQuery function, or equivalent SandboxAPI api.datamartQuery() - both deprecated!
 */
public interface DatamartQueryResult {

	/**
	 * Retreives the query result data.
	 * .
	 * @return The query result data in {@link net.pricefx.formulaengine.scripting.Matrix2D Matrix} format.  All data rows are included
	 *         in this matrix.  The matrix rows represent data rows, while the columns represent the query projections.
	 *         Return <code>null</code> if the maximum number of rows is exceeded. This max is set through Price f(x) instance
	 *         parameter 'datamart.query.externalRowsLimit', with a typical value of 100k rows.
	 */
	Matrix2D getData();

	/**
	 * Additional query summary information,
	 * 
	 * @return A map of maps with (key,value) pairs.
	 */
	Map<String,Serializable> getSummary();
	
    /**
     * Adds the set of distinct values for the specified projections to the Summary map, in summary.distinctValuesFor_<alias>.
     * @param projAliases   The aliases of the projections to inspect
     */
    public void determineDistinctValues(Collection<String> projAliases);
    
    /**
     * Get the distinct values for a given projeciton.  Assumes determineDistinctValues has been run for projection.
     * @param projAlias Alias of a projection 
     * @return  Distinct values for that projection
     */
    public Object getDistinctValues(String projAlias);

    
    /**
     * Calculates basic summary statistics on the query result data, and add it to the Summary map:  
     * Sum, Avg, Mean, Min, Max, Std  
     * and quantiles Q.05, Q.1, Q.2, Q.25, Q.4, Q.5, Q.6, Q.7, Q.75, Q.8, Q.8, Q.95.
     */
    public void calculateSummaryStatistics();	
	
}