package net.pricefx.formulaengine;

import java.sql.ResultSet;
import java.sql.SQLException;
import java.util.Collection;
import java.util.List;
import java.util.Map;
import java.util.concurrent.TimeoutException;

import net.pricefx.formulaengine.scripting.Matrix2D;
import net.pricefx.service.DMUtil.*;


public interface DatamartRowSet {

	public List<String> getKeyFields();

	public Collection<String> getAllFields();

	public boolean isNavigatable();
	
	public Key key(final Object entity, final FieldValues fieldValues );
	
	public Key key(final List<String> keyFields, final FieldValues fieldValues );

	public Key key(final FieldValues fieldValues);
	
	
	/**
	 * Get the row with the given key.  Always returns a copy.
	 * @param key
	 * @return row as FieldValues map
	 */
	public FieldValues getRow(final Key key);

	public FieldValues getRowNoOp(final Key key);
	
	public FieldValues getRow(final FieldValues key);

	public boolean addRow(final Key key, FieldValues row);
	
	public boolean containsRow(final Key key);

	public boolean updateRow(final Key key, FieldValues row);
	
	public boolean deleteRow(final Key key);
	
	public boolean addRow(final FieldValues row);

	// convenience method, typically used in a groovy script 
	public boolean addRow(Map<String,Object> map);

	public boolean updateRow(final FieldValues row);

	// convenience method, typically used in a groovy script 
	public boolean updateRow(Map<String,Object> map);

	public boolean deleteRow(final FieldValues row);

	// convenience method, typically used in a groovy script 
	public boolean deleteRow(Map<String,Object> map);

	// convenience method, typically used in a groovy script 
	public FieldValues getRow(Map<String,Object> map);
	
	public boolean hadAddedRows();
	
	public int getNumberOFRowsWithSurrogateKey();
	
	public FieldValues getDefaultValues();

    public void setDefaultValues(FieldValues defaultValues);
    
    public void initRows(final ResultSet resultSet,
						 Collection<String> fields, 
						 final Object valueSerializer, int timeoutInSecs) throws SQLException, TimeoutException, InterruptedException;
	
	public void loadRows(final DatamartRowSet otherRowset);
	
	public void initRows(final Object queryResult,
						 Collection<String> fields, 
						 final Object valueSerializer, int timeoutInSecs) throws SQLException, TimeoutException;
	

	public void addRows(final Collection<String> fields, 
					    final Collection<FieldValues> rows,
					    final Object valueSerializer) throws TimeoutException;

	public void addRows(final Collection<String> fields, 
						final Collection<FieldValues> rows, 
						final Object valueSerializer, final int timeoutInSecs) throws TimeoutException;

	public boolean isAdded(final FieldValues fv);
	
	public boolean isUpdated(final FieldValues fv);

    public boolean hasSurrogateKey(final FieldValues fv);
	
	public boolean isDirty(final FieldValues fv);

	public FieldValues removeOpField(final FieldValues fv);
	
	public Collection<Key> getKeys();

	public Collection<FieldValues> getRows();

	public boolean isEmpty();
	
	public int size();
	
	public boolean absolute(int idx);

	public boolean relative(int relativeIdx);
	
	public boolean first();

	public boolean last();

	public boolean previous();

	public boolean next();
	
	public FieldValues getCurrentRow();

	public FieldValues getCurrentRowNoOp();

	public boolean updateCurrentRow(final FieldValues row);

	public boolean deleteCurrentRow();
	
	public Key getCurrentKey();

	public Integer getCurrentIndex();

	
	/**
	 * Updates rows in this set, for which a row with the same key exists in the other set.
	 * Does not add any rows to this set!
	 * @param otherSet
	 * @param fields Fields to update in this set
	 */
	public void merge(final DatamartRowSet otherSet);

    public void loadRows(final Matrix2D data);
    
    public void loadRows(final Matrix2D data, final Collection<String> columns);
    
    public void loadRows(final Matrix2D data, final Map<String,String> fieldMapping);

}
