package net.pricefx.formulaengine.scripting.inputbuilder;

import com.fasterxml.jackson.core.JsonProcessingException;
import net.pricefx.common.api.InputType;
import net.pricefx.common.api.PriceFxInterface;
import net.pricefx.common.apibuilder.clicoll.CalculableLineItemCollectionBuilder;
import net.pricefx.lmformulaengine.Util;
import net.pricefx.logic.core.LogicException;
import net.pricefx.server.dto.calculation.ConfiguratorEntry;
import net.pricefx.server.dto.calculation.ContextParameter;

import java.util.List;
import java.util.Map;

public abstract class AbstractInputBuilder<T extends AbstractInputBuilder, VT> {
    protected ContextParameter cp;
    protected final InputBuilderContext builderContext;

    public AbstractInputBuilder(InputBuilderContext builderContext, InputType type, String paramName) {
        this.builderContext = builderContext;
        cp = new ContextParameter(type, paramName,builderContext.getEngineContext().getTargetDate(), null);
    }

    public AbstractInputBuilder(InputBuilderContext builderContext) {
        this.builderContext = builderContext;
    }

    protected Object _getInput() {
        return builderContext.getEngineContext().getParameter(cp.getName());
    }

    public final Object getInput() {
        if(builderContext.getEngineContext().getParameter(cp.getName()) == null)
            builderContext.getEngineContext().addMissingParameter(cp);  //this will effectively prevent the addMissingParameter call in the original functions; hence make sure our amended CP is added

        return _getInput();
    }

    public T setLabel(String label) {
        cp.setLabel(label);
        return (T) this;
    }

    public T setLabelTranslations(Map<String,String> translations) throws LogicException {
        cp.setLabelTranslationsFromMap(translations);
        return (T) this;
    }

    public T setLabelTranslations(Boolean required) {
        cp.setRequired(required);
        return (T) this;
    }

    public T setValue(VT value) {
        cp.setValue(value);
        return (T) this;
    }

    public T setValueHint(String valueHint) {
        cp.setValue(valueHint);
        return (T) this;
    }

    public T setNoRefresh(boolean noRefresh) {
        cp.setConfigParameter("noRefresh", noRefresh);
        return (T) this;
    }

    public T setConfigValues(List<String> configValues) throws LogicException {
        try {
            cp.addParameterConfigEntry(PriceFxInterface.FIELD_CONTEXTPARAM_CONFIG_VALUES, Util.getObjectMapper().writeValueAsString(configValues));
        } catch (JsonProcessingException e) {
            throw new LogicException(e);
        }
        return (T) this;
    }

    public Map<String,Object> buildMap() {
        return Util.getObjectMapper().convertValue(cp, Map.class);
    }

    public ContextParameter buildContextParameter() {
        return cp;
    }

    public T addOrUpdateInput(CalculableLineItemCollectionBuilder clicBuilder, String lineId) {
        if(clicBuilder != null)
            clicBuilder.addOrUpdateInput(lineId,buildMap());
        return (T) this;
    }

    public T addToConfiguratorEntry(ConfiguratorEntry ce) {
        ce.addParameter(cp);
        return (T) this;
    }
}
