package net.pricefx.formulaengine.scripting.inputbuilder;

import net.pricefx.common.api.InputType;
import net.pricefx.common.api.PriceFxInterface;
import net.pricefx.lmformulaengine.LmFormulaEngineContext;

import java.math.BigDecimal;

public class InputBuilderFactory {
    private final InputBuilderContext builderContext;

    public InputBuilderFactory(LmFormulaEngineContext context, Object callback) {
        builderContext = new InputBuilderContext(context,callback);
    }

    public SimpleInputBuilder<BigDecimal> createUserEntry(String paramName) {
        return new SimpleInputBuilder<BigDecimal>(builderContext, paramName, InputType.USERENTRY, null);
    }

    public SimpleInputBuilder<Integer> createIntegerUserEntry(String paramName) {
        return new SimpleInputBuilder<Integer>(builderContext, paramName, InputType.INTEGERUSERENTRY, null);
    }

    public SimpleInputBuilder<Boolean> createBooleanUserEntry(String paramName) {
        return new SimpleInputBuilder<Boolean>(builderContext, paramName, InputType.BOOLEANUSERENTRY, null);
    }

    public SimpleInputBuilder<String> createDateUserEntry(String paramName) {
        return new SimpleInputBuilder<String>(builderContext, paramName, InputType.DATEUSERENTRY, null);
    }

    public DateRangeInputBuilder createDateRangeUserEntry(String paramName) {
        return new DateRangeInputBuilder(builderContext, paramName);
    }

    public SimpleInputBuilder<String> createTimeUserEntry(String paramName) {
        return new SimpleInputBuilder<String>(builderContext, paramName, InputType.TIMEUSERENTRY, null);
    }

    public SimpleInputBuilder<String> createDateTimeUserEntry(String paramName) {
        return new SimpleInputBuilder<String>(builderContext, paramName, InputType.DATETIMEUSERENTRY,null);
    }


    public OptionInputBuilder createRadioEntry(String paramName) {
        return new OptionInputBuilder(builderContext, InputType.RADIO, paramName);
    }

    public SimpleInputBuilder<Object> createHiddenEntry(String paramName) {
        return new SimpleInputBuilder<Object>(builderContext, paramName, InputType.HIDDEN, null);
    }

    public StringInputBuilder<String> createStringUserEntry(String paramName) {
        return new StringInputBuilder<String>(builderContext, paramName);
    }

    public SimpleInputBuilder<String> createTextUserEntry(String paramName) {
        return new SimpleInputBuilder<String>(builderContext, paramName, InputType.TEXTUSERENTRY, null);
    }

    public SimpleInputBuilder<String> createAnyUserEntry(String paramName) {
        return new SimpleInputBuilder<String>(builderContext, paramName, InputType.USER, null);
    }

    public VLookupBuilder createVLookup(String paramName) {
        return new VLookupBuilder(builderContext, paramName);
    }

    public ProductInputBuilder createProductEntry() {
        return new ProductInputBuilder(builderContext, PriceFxInterface.CONTEXTPARAM_NAME_PRODUCTSKU);
    }

    public ProductInputBuilder createProductEntry(String paramName) {
        return new ProductInputBuilder(builderContext, paramName);
    }

    public PCGroupInputBuilder createProductGroupEntry() {
        return new PCGroupInputBuilder(builderContext, InputType.PRODUCTGROUP, PriceFxInterface.CONTEXTPARAM_NAME_PRODUCTGROUP);
    }

    public PCGroupInputBuilder createProductGroupEntry(String paramName) {
        return new PCGroupInputBuilder(builderContext, InputType.PRODUCTGROUP, paramName);
    }

    public CustomerInputBuilder createCustomerEntry() {
        return new CustomerInputBuilder(builderContext, PriceFxInterface.CONTEXTPARAM_NAME_CUSTOMER);
    }

    public CustomerInputBuilder createCustomerEntry(String paramName) {
        return new CustomerInputBuilder(builderContext, paramName);
    }

    public PCGroupInputBuilder createCustomerGroupEntry() {
        return new PCGroupInputBuilder(builderContext, InputType.CUSTOMERGROUP, PriceFxInterface.CONTEXTPARAM_NAME_CUSTOMERGROUP);
    }

    public PCGroupInputBuilder createCustomerGroupEntry(String paramName) {
        return new PCGroupInputBuilder(builderContext, InputType.CUSTOMERGROUP, paramName);
    }

    public OptionInputBuilder createOptionEntry(String paramName) {
        return new OptionInputBuilder(builderContext, InputType.OPTION, paramName);
    }

    public OptionInputBuilder createOptionsEntry(String paramName) {
        return new OptionInputBuilder(builderContext, InputType.OPTIONS, paramName);
    }

    public InputMatrixInputBuilder createInputMatrix(String paramName) {
        return new InputMatrixInputBuilder(builderContext, paramName);
    }

    public FilterBuilderInputBuilder createFilterBuilder(String paramName, String typeCode) {
        return new FilterBuilderInputBuilder(builderContext, paramName, typeCode);
    }

    public ConfiguratorInputBuilder createFilterBuilder(String paramName, String formulaName, boolean isInline) {
        return new ConfiguratorInputBuilder(builderContext, paramName, formulaName, isInline);
    }

    public DmFilterBuilder createDmFilter(String paramName, String fcTypedId, String field) {
        return new DmFilterBuilder(builderContext, paramName, fcTypedId, field);
    }


}
