package net.pricefx.lmformulaengine.model;

import groovy.lang.Binding;
import groovy.lang.GroovyShell;
import groovy.lang.Script;
import net.pricefx.lmformulaengine.LmFormulaEngineContext;
import net.pricefx.lmformulaengine.scripting.LmSandboxAPI;
import net.pricefx.logic.core.AbstractLogicSerializer;
import net.pricefx.logic.core.LogicException;
import net.pricefx.logic.core.LogicSerializer;
import net.pricefx.logic.core.model.AbstractFormula;
import net.pricefx.logic.core.model.AbstractFormulaElement;
import net.pricefx.pckg.transform.TransformCalculationLogic;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.io.IOException;
import java.nio.file.Path;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Map;

import static net.pricefx.lmformulaengine.LmDefaultFormulaEngine.VAR_api;
import static net.pricefx.lmformulaengine.LmDefaultFormulaEngine.VAR_libs;

/**
 * simulate libs shortcut
 */
public class LibraryBindingMap extends HashMap<String, Map<String, Script>>  {
    private static final Logger log = LoggerFactory.getLogger(LibraryBindingMap.class);
    private Path moduleDir;
    private GroovyShell groovyShell;
    private LmSandboxAPI api;
    private LmFormulaEngineContext ctx;

    public LibraryBindingMap(Map<String, Map<String, Script>> value, Path moduleDir, GroovyShell groovyShell, LmSandboxAPI api, LmFormulaEngineContext ctx) {
        super(value);
        this.moduleDir = moduleDir;
        this.groovyShell = groovyShell;
        this.api = api;
        this.ctx = ctx;
    }

    private void loadIfNotLoaded(Object key) {
        if (key == null) {
            return;
        }

        if (!super.containsKey(key.toString())) {
            Path calculationLogicDir = moduleDir.resolve(TransformCalculationLogic.DIRNAME);

            try {
                List<String> matchedLibraries = AbstractLogicSerializer.getAllMatchedLibraries(calculationLogicDir, key.toString());
                AbstractFormula formula = AbstractLogicSerializer.getLatestLibrary(moduleDir, key.toString(), matchedLibraries);
                if (formula != null) {
                    super.put(key.toString(), getElementsMap(formula, groovyShell, api));
                }
            } catch (IOException e) {
                log.error(e.getMessage(), e);
            }
        }
    }

    @Override
    public Map<String, Script> get(Object key) {
        loadIfNotLoaded(key);
        return super.get(key);
    }

    @Override
    public boolean containsKey(Object key) {
        loadIfNotLoaded(key);
        return super.containsKey(key);
    }

    /**
     * Simulating LmDefaultFormulaEngine.initializeLibrary(). should be kept in sync with core
     * @param groovyLogic
     * @param groovyShell
     * @param api
     * @return
     * @throws LogicException
     * @throws IOException
     */
    private Map<String, Script> getElementsMap(AbstractFormula groovyLogic, GroovyShell groovyShell, LmSandboxAPI api) throws IOException {

        Map<String, Script> elementsMap = new HashMap<>();

        Iterator<AbstractFormulaElement> it = groovyLogic.getElementsIterator();

        while (it.hasNext()) {
            AbstractFormulaElement element = it.next();
            Script groovyObj = groovyShell.parse(LogicSerializer.getElementPath(groovyLogic.getDirPath(), element).toFile());

            //make api and libs available to library objects
            Binding binding = new Binding();
            binding.setVariable(VAR_api, api);
            binding.setVariable(VAR_libs, this);

            if(ctx.getGroovyBindings() != null) {
                ctx.getGroovyBindings().forEach( (k,v) -> binding.setVariable(k,v) );
            }

            groovyObj.setBinding(binding);

            elementsMap.put(element.getElementName(), groovyObj);

        }

        return elementsMap;

    }

}
